#[cfg(feature = "Win32_Devices_Properties")]
pub const DEVPKEY_Bluetooth_ClassOfDevice: super::super::super::Win32::Devices::Properties::DEVPROPKEY = super::super::super::Win32::Devices::Properties::DEVPROPKEY { fmtid: windows_sys::core::GUID::from_u128(0x2bd67d8b_8beb_48d5_87e0_6cda3428040a), pid: 10 };
#[cfg(feature = "Win32_Devices_Properties")]
pub const DEVPKEY_Bluetooth_ClassOfDevice_Deprecated: super::super::super::Win32::Devices::Properties::DEVPROPKEY = super::super::super::Win32::Devices::Properties::DEVPROPKEY { fmtid: windows_sys::core::GUID::from_u128(0x2bd67d8b_8beb_48d5_87e0_6cda3428040a), pid: 4 };
#[cfg(feature = "Win32_Devices_Properties")]
pub const DEVPKEY_Bluetooth_DeviceAddress: super::super::super::Win32::Devices::Properties::DEVPROPKEY = super::super::super::Win32::Devices::Properties::DEVPROPKEY { fmtid: windows_sys::core::GUID::from_u128(0x2bd67d8b_8beb_48d5_87e0_6cda3428040a), pid: 1 };
#[cfg(feature = "Win32_Devices_Properties")]
pub const DEVPKEY_Bluetooth_DeviceFlags: super::super::super::Win32::Devices::Properties::DEVPROPKEY = super::super::super::Win32::Devices::Properties::DEVPROPKEY { fmtid: windows_sys::core::GUID::from_u128(0x2bd67d8b_8beb_48d5_87e0_6cda3428040a), pid: 3 };
#[cfg(feature = "Win32_Devices_Properties")]
pub const DEVPKEY_Bluetooth_DeviceManufacturer: super::super::super::Win32::Devices::Properties::DEVPROPKEY = super::super::super::Win32::Devices::Properties::DEVPROPKEY { fmtid: windows_sys::core::GUID::from_u128(0x2bd67d8b_8beb_48d5_87e0_6cda3428040a), pid: 4 };
#[cfg(feature = "Win32_Devices_Properties")]
pub const DEVPKEY_Bluetooth_DeviceModelNumber: super::super::super::Win32::Devices::Properties::DEVPROPKEY = super::super::super::Win32::Devices::Properties::DEVPROPKEY { fmtid: windows_sys::core::GUID::from_u128(0x2bd67d8b_8beb_48d5_87e0_6cda3428040a), pid: 5 };
#[cfg(feature = "Win32_Devices_Properties")]
pub const DEVPKEY_Bluetooth_DevicePID: super::super::super::Win32::Devices::Properties::DEVPROPKEY = super::super::super::Win32::Devices::Properties::DEVPROPKEY { fmtid: windows_sys::core::GUID::from_u128(0x2bd67d8b_8beb_48d5_87e0_6cda3428040a), pid: 8 };
#[cfg(feature = "Win32_Devices_Properties")]
pub const DEVPKEY_Bluetooth_DeviceProductVersion: super::super::super::Win32::Devices::Properties::DEVPROPKEY = super::super::super::Win32::Devices::Properties::DEVPROPKEY { fmtid: windows_sys::core::GUID::from_u128(0x2bd67d8b_8beb_48d5_87e0_6cda3428040a), pid: 9 };
#[cfg(feature = "Win32_Devices_Properties")]
pub const DEVPKEY_Bluetooth_DeviceVID: super::super::super::Win32::Devices::Properties::DEVPROPKEY = super::super::super::Win32::Devices::Properties::DEVPROPKEY { fmtid: windows_sys::core::GUID::from_u128(0x2bd67d8b_8beb_48d5_87e0_6cda3428040a), pid: 7 };
#[cfg(feature = "Win32_Devices_Properties")]
pub const DEVPKEY_Bluetooth_DeviceVIDSource: super::super::super::Win32::Devices::Properties::DEVPROPKEY = super::super::super::Win32::Devices::Properties::DEVPROPKEY { fmtid: windows_sys::core::GUID::from_u128(0x2bd67d8b_8beb_48d5_87e0_6cda3428040a), pid: 6 };
#[cfg(feature = "Win32_Devices_Properties")]
pub const DEVPKEY_Bluetooth_LastConnectedTime: super::super::super::Win32::Devices::Properties::DEVPROPKEY = super::super::super::Win32::Devices::Properties::DEVPROPKEY { fmtid: windows_sys::core::GUID::from_u128(0x2bd67d8b_8beb_48d5_87e0_6cda3428040a), pid: 11 };
#[cfg(feature = "Win32_Devices_Properties")]
pub const DEVPKEY_Bluetooth_LastConnectedTime_Deprecated: super::super::super::Win32::Devices::Properties::DEVPROPKEY = super::super::super::Win32::Devices::Properties::DEVPROPKEY { fmtid: windows_sys::core::GUID::from_u128(0x2bd67d8b_8beb_48d5_87e0_6cda3428040a), pid: 5 };
#[cfg(feature = "Win32_Devices_Properties")]
pub const DEVPKEY_Bluetooth_LastSeenTime: super::super::super::Win32::Devices::Properties::DEVPROPKEY = super::super::super::Win32::Devices::Properties::DEVPROPKEY { fmtid: windows_sys::core::GUID::from_u128(0x2bd67d8b_8beb_48d5_87e0_6cda3428040a), pid: 12 };
#[cfg(feature = "Win32_Devices_Properties")]
pub const DEVPKEY_Bluetooth_ServiceGUID: super::super::super::Win32::Devices::Properties::DEVPROPKEY = super::super::super::Win32::Devices::Properties::DEVPROPKEY { fmtid: windows_sys::core::GUID::from_u128(0x2bd67d8b_8beb_48d5_87e0_6cda3428040a), pid: 2 };
pub const GUID_BTHDDI_PROFILE_DRIVER_INTERFACE: windows_sys::core::GUID = windows_sys::core::GUID::from_u128(0x94a59aa8_4383_4286_aa4f_34a160f40004);
pub const GUID_BTHDDI_SDP_NODE_INTERFACE: windows_sys::core::GUID = windows_sys::core::GUID::from_u128(0x81a7fdf3_86c1_4be8_a8c8_2a6d188b4177);
pub const GUID_BTHDDI_SDP_PARSE_INTERFACE: windows_sys::core::GUID = windows_sys::core::GUID::from_u128(0x4e719439_9cf1_4bab_ac1d_3279865743d2);
pub const GUID_BTH_DEVICE_INTERFACE: windows_sys::core::GUID = windows_sys::core::GUID::from_u128(0x00f40965_e89d_4487_9890_87c3abb211f4);
