pub const MSG_category_Devices: i32 = 1i32;
pub const MSG_category_Disk: i32 = 2i32;
pub const MSG_category_Network: i32 = 7i32;
pub const MSG_category_Printers: i32 = 3i32;
pub const MSG_category_Services: i32 = 4i32;
pub const MSG_category_Shell: i32 = 5i32;
pub const MSG_category_SystemEvent: i32 = 6i32;
pub const MSG_channel_Application: i32 = 256i32;
pub const MSG_channel_ProviderMetadata: i32 = -1879048189i32;
pub const MSG_channel_Security: i32 = 257i32;
pub const MSG_channel_System: i32 = 258i32;
pub const MSG_channel_TraceClassic: i32 = -1879048191i32;
pub const MSG_channel_TraceLogging: i32 = -1879048190i32;
pub const MSG_keyword_AnyKeyword: i32 = 268435456i32;
pub const MSG_keyword_AuditFailure: i32 = 268435509i32;
pub const MSG_keyword_AuditSuccess: i32 = 268435510i32;
pub const MSG_keyword_Classic: i32 = 268435512i32;
pub const MSG_keyword_CorrelationHint: i32 = 268435511i32;
pub const MSG_keyword_ResponseTime: i32 = 268435505i32;
pub const MSG_keyword_SQM: i32 = 268435508i32;
pub const MSG_keyword_WDIDiag: i32 = 268435507i32;
pub const MSG_level_Critical: i32 = 1342177281i32;
pub const MSG_level_Error: i32 = 1342177282i32;
pub const MSG_level_Informational: i32 = 1342177284i32;
pub const MSG_level_LogAlways: i32 = 1342177280i32;
pub const MSG_level_Verbose: i32 = 1342177285i32;
pub const MSG_level_Warning: i32 = 1342177283i32;
pub const MSG_opcode_DCStart: i32 = 805502976i32;
pub const MSG_opcode_DCStop: i32 = 805568512i32;
pub const MSG_opcode_Extension: i32 = 805634048i32;
pub const MSG_opcode_Info: i32 = 805306368i32;
pub const MSG_opcode_Receive: i32 = 821035008i32;
pub const MSG_opcode_Reply: i32 = 805699584i32;
pub const MSG_opcode_Resume: i32 = 805765120i32;
pub const MSG_opcode_Send: i32 = 805896192i32;
pub const MSG_opcode_Start: i32 = 805371904i32;
pub const MSG_opcode_Stop: i32 = 805437440i32;
pub const MSG_opcode_Suspend: i32 = 805830656i32;
pub const MSG_task_None: i32 = 1879048192i32;
pub const WINEVENT_CHANNEL_CLASSIC_TRACE: i32 = 0i32;
pub const WINEVENT_CHANNEL_GLOBAL_APPLICATION: i32 = 9i32;
pub const WINEVENT_CHANNEL_GLOBAL_SECURITY: i32 = 10i32;
pub const WINEVENT_CHANNEL_GLOBAL_SYSTEM: i32 = 8i32;
pub const WINEVENT_CHANNEL_PROVIDERMETADATA: i32 = 12i32;
pub const WINEVENT_CHANNEL_TRACELOGGING: i32 = 11i32;
pub const WINEVENT_KEYWORD_AUDIT_FAILURE: i64 = 4503599627370496i64;
pub const WINEVENT_KEYWORD_AUDIT_SUCCESS: i64 = 9007199254740992i64;
pub const WINEVENT_KEYWORD_CORRELATION_HINT: i64 = 18014398509481984i64;
pub const WINEVENT_KEYWORD_EVENTLOG_CLASSIC: i64 = 36028797018963968i64;
pub const WINEVENT_KEYWORD_RESERVED_49: i64 = 562949953421312i64;
pub const WINEVENT_KEYWORD_RESERVED_56: i64 = 72057594037927936i64;
pub const WINEVENT_KEYWORD_RESERVED_57: i64 = 144115188075855872i64;
pub const WINEVENT_KEYWORD_RESERVED_58: i64 = 288230376151711744i64;
pub const WINEVENT_KEYWORD_RESERVED_59: i64 = 576460752303423488i64;
pub const WINEVENT_KEYWORD_RESERVED_60: i64 = 1152921504606846976i64;
pub const WINEVENT_KEYWORD_RESERVED_61: i64 = 2305843009213693952i64;
pub const WINEVENT_KEYWORD_RESERVED_62: i64 = 4611686018427387904i64;
pub const WINEVENT_KEYWORD_RESERVED_63: u64 = 9223372036854775808u64;
pub const WINEVENT_KEYWORD_RESPONSE_TIME: i64 = 281474976710656i64;
pub const WINEVENT_KEYWORD_SQM: i64 = 2251799813685248i64;
pub const WINEVENT_KEYWORD_WDI_DIAG: i64 = 1125899906842624i64;
pub const WINEVENT_LEVEL_CRITICAL: i32 = 1i32;
pub const WINEVENT_LEVEL_ERROR: i32 = 2i32;
pub const WINEVENT_LEVEL_INFO: i32 = 4i32;
pub const WINEVENT_LEVEL_LOG_ALWAYS: i32 = 0i32;
pub const WINEVENT_LEVEL_RESERVED_10: i32 = 10i32;
pub const WINEVENT_LEVEL_RESERVED_11: i32 = 11i32;
pub const WINEVENT_LEVEL_RESERVED_12: i32 = 12i32;
pub const WINEVENT_LEVEL_RESERVED_13: i32 = 13i32;
pub const WINEVENT_LEVEL_RESERVED_14: i32 = 14i32;
pub const WINEVENT_LEVEL_RESERVED_15: i32 = 15i32;
pub const WINEVENT_LEVEL_RESERVED_6: i32 = 6i32;
pub const WINEVENT_LEVEL_RESERVED_7: i32 = 7i32;
pub const WINEVENT_LEVEL_RESERVED_8: i32 = 8i32;
pub const WINEVENT_LEVEL_RESERVED_9: i32 = 9i32;
pub const WINEVENT_LEVEL_VERBOSE: i32 = 5i32;
pub const WINEVENT_LEVEL_WARNING: i32 = 3i32;
pub const WINEVENT_OPCODE_DC_START: i32 = 3i32;
pub const WINEVENT_OPCODE_DC_STOP: i32 = 4i32;
pub const WINEVENT_OPCODE_EXTENSION: i32 = 5i32;
pub const WINEVENT_OPCODE_INFO: i32 = 0i32;
pub const WINEVENT_OPCODE_RECEIVE: i32 = 240i32;
pub const WINEVENT_OPCODE_REPLY: i32 = 6i32;
pub const WINEVENT_OPCODE_RESERVED_241: i32 = 241i32;
pub const WINEVENT_OPCODE_RESERVED_242: i32 = 242i32;
pub const WINEVENT_OPCODE_RESERVED_243: i32 = 243i32;
pub const WINEVENT_OPCODE_RESERVED_244: i32 = 244i32;
pub const WINEVENT_OPCODE_RESERVED_245: i32 = 245i32;
pub const WINEVENT_OPCODE_RESERVED_246: i32 = 246i32;
pub const WINEVENT_OPCODE_RESERVED_247: i32 = 247i32;
pub const WINEVENT_OPCODE_RESERVED_248: i32 = 248i32;
pub const WINEVENT_OPCODE_RESERVED_249: i32 = 249i32;
pub const WINEVENT_OPCODE_RESERVED_250: i32 = 250i32;
pub const WINEVENT_OPCODE_RESERVED_251: i32 = 251i32;
pub const WINEVENT_OPCODE_RESERVED_252: i32 = 252i32;
pub const WINEVENT_OPCODE_RESERVED_253: i32 = 253i32;
pub const WINEVENT_OPCODE_RESERVED_254: i32 = 254i32;
pub const WINEVENT_OPCODE_RESERVED_255: i32 = 255i32;
pub const WINEVENT_OPCODE_RESUME: i32 = 7i32;
pub const WINEVENT_OPCODE_SEND: i32 = 9i32;
pub const WINEVENT_OPCODE_START: i32 = 1i32;
pub const WINEVENT_OPCODE_STOP: i32 = 2i32;
pub const WINEVENT_OPCODE_SUSPEND: i32 = 8i32;
pub const WINEVENT_TASK_NONE: i32 = 0i32;
pub const WINEVT_KEYWORD_ANY: i32 = 0i32;
